@@ -33,7 +33,7 @@ class BaseBLAS(COp):
 
 
 def ger_c_code(A, a, x, y, Z, fail, params):
-    return """
+    return f"""
 
     int elemsize ;
 
@@ -309,7 +309,7 @@ def ger_c_code(A, a, x, y, Z, fail, params):
         }}
     }}
 
-    """.format(**locals())
+    """
 
 
 class CGer(BaseBLAS, Ger):
@@ -336,14 +336,16 @@ cger_no_inplace = CGer(False)
 # ##### ####### #######
 
 
-def gemv_c_code(y, A, x, z, alpha, beta, fail, force_init_beta=False, params=None):
+def gemv_c_code(y, A, x, z, alpha, beta, fail, must_initialize_y=False, params=None):
     """
     z <- beta * y + alpha * dot(A, x)
 
     where A is a matrix, y and x are vectors (ergo z is vector)
+    z = y if inplace else y.copy()
     """
     code = """
 
+    bool is_float;
     int elemsize;
     float fbeta;
     double dbeta;
@@ -361,12 +363,24 @@ def gemv_c_code(y, A, x, z, alpha, beta, fail, force_init_beta=False, params=Non
         %(fail)s;
     }
 
-    if  (PyArray_DESCR(%(y)s)->type_num == NPY_DOUBLE) { elemsize = 8; }
-    else if (PyArray_DESCR(%(y)s)->type_num == NPY_FLOAT) { elemsize = 4;}
-    else {
-        PyErr_SetString(PyExc_NotImplementedError, "complex Gemv");
+    if ((PyArray_DESCR(%(y)s)->type_num != PyArray_DESCR(%(x)s)->type_num)
+        || (PyArray_DESCR(%(y)s)->type_num != PyArray_DESCR(%(A)s)->type_num))
+    {
+        PyErr_SetString(PyExc_TypeError, "GEMV: dtypes of A, x, y do not match");
         %(fail)s;
     }
+    if  (PyArray_DESCR(%(y)s)->type_num == NPY_DOUBLE) {
+        is_float = 0;
+        elemsize = 8;
+    }
+    else if (PyArray_DESCR(%(y)s)->type_num == NPY_FLOAT) {
+        elemsize = 4;
+        is_float = 1;
+    }
+    else {
+        %(fail)s;
+        PyErr_SetString(PyExc_NotImplementedError, "GEMV: Inputs must be float or double");
+    }
 
     fbeta = dbeta = ((dtype_%(beta)s*)PyArray_DATA(%(beta)s))[0];
 
@@ -387,17 +401,11 @@ def gemv_c_code(y, A, x, z, alpha, beta, fail, force_init_beta=False, params=Non
         }
         if (dbeta != 0)
         {
+            // If dbeta is zero, we avoid doing the copy
             if (PyArray_CopyInto(%(z)s, %(y)s) != 0) {
                 %(fail)s
             }
         }
-        else if (%(force_init_beta)d)
-        {
-            PyObject *zero = PyFloat_FromDouble(0.);
-            if (zero == NULL) %(fail)s;
-            if (PyArray_FillWithScalar(%(z)s, zero) != 0) %(fail)s;
-            Py_DECREF(zero);
-        }
     }
     else
     {
@@ -408,171 +416,158 @@ def gemv_c_code(y, A, x, z, alpha, beta, fail, force_init_beta=False, params=Non
             Py_INCREF(%(z)s);
         }
     }
+
+    if (%(must_initialize_y)d && dbeta == 0)
+    {
+        // Most BLAS implementations of GEMV ignore y=nan when beta=0
+        // PyTensor considers that the correct behavior,
+        // and even exploits it to avoid copying or initializing outputs.
+        // By deciding to exploit this, however, it becomes our responsibility
+        // to ensure the behavior even in the rare cases BLAS deviates,
+        // or users will get errors, even for graphs that had no nan to begin with.
+        PyObject *zero = PyFloat_FromDouble(0.);
+        if (zero == NULL) %(fail)s;
+        if (PyArray_FillWithScalar(%(z)s, zero) != 0) %(fail)s;
+        Py_DECREF(zero);
+    }
+
     {
-        char TRANS = 'T';
-        char NOTRANS = 'N';
         int NA0 = PyArray_DIMS(%(A)s)[0];
         int NA1 = PyArray_DIMS(%(A)s)[1];
-        /* This formula is needed in the case where A is actually a row or
-         * column matrix, because BLAS sometimes insists that the strides:
-         *  - are not smaller than the number of elements in the array
-         *  - are not 0.
-         */
-        int SA0 = (NA0 > 1) ? (PyArray_STRIDES(%(A)s)[0] / elemsize) : (NA1 + 1);
-        int SA1 = (NA1 > 1) ? (PyArray_STRIDES(%(A)s)[1] / elemsize) : (NA0 + 1);
-        int Sz = PyArray_STRIDES(%(z)s)[0] / elemsize;
-        int Sx = PyArray_STRIDES(%(x)s)[0] / elemsize;
-
-        dtype_%(x)s* x_data = (dtype_%(x)s*) PyArray_DATA(%(x)s);
-        dtype_%(z)s* z_data = (dtype_%(z)s*) PyArray_DATA(%(z)s);
-        // gemv expects pointers to the beginning of memory arrays,
-        // but numpy provides a pointer to the first element,
-        // so when the stride is negative, we need to get the last one.
-        if (Sx < 0)
-            x_data += (NA1 - 1) * Sx;
-        if (Sz < 0)
-            z_data += (NA0 - 1) * Sz;
 
         if (NA0 * NA1)
         {
-            // If A is neither C- nor F-contiguous, we make a copy.
-            // TODO:
-            // - if one stride is equal to "- elemsize", we can still call
-            //   gemv on reversed matrix and vectors
-            // - if the copy is too long, maybe call vector/vector dot on
-            //   each row instead
-            if ((PyArray_STRIDES(%(A)s)[0] < 0)
-                || (PyArray_STRIDES(%(A)s)[1] < 0)
-                || ((PyArray_STRIDES(%(A)s)[0] != elemsize)
-                    && (PyArray_STRIDES(%(A)s)[1] != elemsize)))
+            // Non-empty A matrix
+
+            /* In the case where A is actually a row or column matrix,
+             * the strides corresponding to the dummy dimension don't matter,
+             * but BLAS requires these to be no smaller than the number of elements in the array.
+             */
+            int SA0 = (NA0 > 1) ? (PyArray_STRIDES(%(A)s)[0] / elemsize) : NA1;
+            int SA1 = (NA1 > 1) ? (PyArray_STRIDES(%(A)s)[1] / elemsize) : NA0;
+            int Sz = PyArray_STRIDES(%(z)s)[0] / elemsize;
+            int Sx = PyArray_STRIDES(%(x)s)[0] / elemsize;
+
+            dtype_%(A)s* A_data = (dtype_%(A)s*) PyArray_DATA(%(A)s);
+            dtype_%(x)s* x_data = (dtype_%(x)s*) PyArray_DATA(%(x)s);
+            dtype_%(z)s* z_data = (dtype_%(z)s*) PyArray_DATA(%(z)s);
+
+            // gemv expects pointers to the beginning of memory arrays,
+            // but numpy provides a pointer to the first element,
+            // so when the stride is negative, we need to get the last one.
+            if (Sx < 0)
+                x_data += (NA1 - 1) * Sx;
+            if (Sz < 0)
+                z_data += (NA0 - 1) * Sz;
+
+            if ( ((SA0 < 0) || (SA1 < 0)) && (abs(SA0) == 1 || (abs(SA1) == 1)) )
             {
+                // We can treat the array A as C-or F-contiguous by changing the order of iteration
+                // printf("GEMV: Iterating in reverse NA0=%%d, NA1=%%d, SA0=%%d, SA1=%%d\\n", NA0, NA1, SA0, SA1);
+                if (SA0 < 0){
+                    A_data += (NA0 -1) * SA0;  // Jump to first row
+                    SA0 = -SA0;  // Iterate over rows in reverse
+                    Sz = -Sz;  // Iterate over y in reverse
+                }
+                if (SA1 < 0){
+                    A_data += (NA1 -1) * SA1;  // Jump to first column
+                    SA1 = -SA1;  // Iterate over columns in reverse
+                    Sx = -Sx;  // Iterate over x in reverse
+                }
+            } else if ((SA0 < 0) || (SA1 < 0) || ((SA0 != 1) && (SA1 != 1)))
+            {
+                // Array isn't contiguous, we have to make a copy
+                // - if the copy is too long, maybe call vector/vector dot on each row instead
+                // printf("GEMV: Making a copy NA0=%%d, NA1=%%d, SA0=%%d, SA1=%%d\\n", NA0, NA1, SA0, SA1);
                 npy_intp dims[2];
                 dims[0] = NA0;
                 dims[1] = NA1;
-
-                PyArrayObject * A_copy = (PyArrayObject *) PyArray_Copy(
-                                                                   %(A)s);
+                PyArrayObject * A_copy = (PyArrayObject *) PyArray_Copy(%(A)s);
                 if (!A_copy)
                     %(fail)s
                 Py_XDECREF(%(A)s);
                 %(A)s = A_copy;
-                SA0 = (NA0 > 1) ? (PyArray_STRIDES(%(A)s)[0] / elemsize) : (NA1 + 1);
-                SA1 = (NA1 > 1) ? (PyArray_STRIDES(%(A)s)[1] / elemsize) : (NA0 + 1);
+                SA0 = (NA0 > 1) ? (PyArray_STRIDES(%(A)s)[0] / elemsize) : NA1;
+                SA1 = (NA1 > 1) ? (PyArray_STRIDES(%(A)s)[1] / elemsize) : NA0;
+                A_data = (dtype_%(A)s*) PyArray_DATA(%(A)s);
             }
+            //else {printf("GEMV: Using the original array NA0=%%d, NA1=%%d, SA0=%%d, SA1=%%d\\n", NA0, NA1, SA0, SA1);}
 
-            if (PyArray_STRIDES(%(A)s)[0] == elemsize)
+            if (NA0 == 1)
             {
-                if (PyArray_DESCR(%(A)s)->type_num == NPY_FLOAT)
+                // Vector-vector dot product, it seems faster to avoid GEMV
+                dtype_%(alpha)s alpha = ((dtype_%(alpha)s*)PyArray_DATA(%(alpha)s))[0];
+
+                if (is_float)
+                {
+                    z_data[0] = dbeta != 0 ? dbeta * z_data[0] : 0.f;
+                    z_data[0] += alpha * sdot_(&NA1,  (float*)(A_data), &SA1,
+                                              (float*)x_data, &Sx);
+                }
+                else
+                {
+                    z_data[0] = dbeta != 0 ? dbeta * z_data[0] : 0.;
+                    z_data[0] += alpha * ddot_(&NA1,  (double*)(A_data), &SA1,
+                                              (double*)x_data, &Sx);
+                }
+            }
+            else if (SA0 == 1)
+            {
+                // F-contiguous
+                char NOTRANS = 'N';
+                if (is_float)
                 {
                     float alpha = ((dtype_%(alpha)s*)PyArray_DATA(%(alpha)s))[0];
                     sgemv_(&NOTRANS, &NA0, &NA1,
                         &alpha,
-                        (float*)(PyArray_DATA(%(A)s)), &SA1,
+                        (float*)(A_data), &SA1,
                         (float*)x_data, &Sx,
                         &fbeta,
                         (float*)z_data, &Sz);
                 }
-                else if (PyArray_DESCR(%(A)s)->type_num == NPY_DOUBLE)
+                else
                 {
                     double alpha = ((dtype_%(alpha)s*)PyArray_DATA(%(alpha)s))[0];
                     dgemv_(&NOTRANS, &NA0, &NA1,
                         &alpha,
-                        (double*)(PyArray_DATA(%(A)s)), &SA1,
+                        (double*)(A_data), &SA1,
                         (double*)x_data, &Sx,
                         &dbeta,
                         (double*)z_data, &Sz);
                 }
-                else
-                {
-                    PyErr_SetString(PyExc_AssertionError,
-                                    "neither float nor double dtype");
-                    %(fail)s
-                }
             }
-            else if (PyArray_STRIDES(%(A)s)[1] == elemsize)
+            else if (SA1 == 1)
             {
-                if (PyArray_DESCR(%(A)s)->type_num == NPY_FLOAT)
+                // C-contiguous
+                char TRANS = 'T';
+                if (is_float)
                 {
                     float alpha = ((dtype_%(alpha)s*)PyArray_DATA(%(alpha)s))[0];
-
-                    // Check for vector-vector dot (NA0 == 1). The code may work
-                    // for SA1 != 1 as well, but has not been tested for this case,
-                    // so SA1 == 1 is required for safety.
-                    if (NA0 == 1 && SA1 == 1)
-                    {
-                        if (fbeta != 0.f) {
-                          z_data[0] = fbeta*z_data[0];
-                        } else {
-                          z_data[0] = 0.f;
-                        }
-                        z_data[0] += alpha*sdot_(&NA1,
-                              (float*)(PyArray_DATA(%(A)s)), &SA1,
-                              (float*)x_data, &Sx);
-                    }
-                    else
-                    {
-                        sgemv_(&TRANS, &NA1, &NA0,
-                            &alpha,
-                            (float*)(PyArray_DATA(%(A)s)), &SA0,
-                            (float*)x_data, &Sx,
-                            &fbeta,
-                            (float*)z_data, &Sz);
-                    }
-                }
-                else if (PyArray_DESCR(%(A)s)->type_num == NPY_DOUBLE)
-                {
-                    double alpha = ((dtype_%(alpha)s*)PyArray_DATA(%(alpha)s))[0];
-
-                    // Check for vector-vector dot (NA0 == 1). The code may work
-                    // for SA1 != 1 as well, but has not been tested for this case,
-                    // so SA1 == 1 is required for safety.
-                    if (NA0 == 1 && SA1 == 1)
-                    {
-                        if (dbeta != 0.) {
-                          z_data[0] = dbeta*z_data[0];
-                        } else {
-                          z_data[0] = 0.;
-                        }
-                        z_data[0] += alpha*ddot_(&NA1,
-                              (double*)(PyArray_DATA(%(A)s)), &SA1,
-                              (double*)x_data, &Sx);
-                    }
-                    else
-                    {
-                        dgemv_(&TRANS, &NA1, &NA0,
-                            &alpha,
-                            (double*)(PyArray_DATA(%(A)s)), &SA0,
-                            (double*)x_data, &Sx,
-                            &dbeta,
-                            (double*)z_data, &Sz);
-                    }
+                    sgemv_(&TRANS, &NA1, &NA0,
+                        &alpha,
+                        (float*)(A_data), &SA0,
+                        (float*)x_data, &Sx,
+                        &fbeta,
+                        (float*)z_data, &Sz);
                 }
                 else
                 {
-                    PyErr_SetString(PyExc_AssertionError,
-                                    "neither float nor double dtype");
-                    %(fail)s
+                    double alpha = ((dtype_%(alpha)s*)PyArray_DATA(%(alpha)s))[0];
+                    dgemv_(&TRANS, &NA1, &NA0,
+                        &alpha,
+                        (double*)(A_data), &SA0,
+                        (double*)x_data, &Sx,
+                        &dbeta,
+                        (double*)z_data, &Sz);
                 }
             }
             else
             {
                 PyErr_SetString(PyExc_AssertionError,
-                    "xx is a double-strided matrix, and should have been "
-                    "copied into a memory-contiguous one.");
+                                "A is neither C nor F-contiguous, it should have been copied into a memory-contiguous array;");
                 %(fail)s
             }
         }
-        else if (dbeta != 1.0)
-        {
-            // the matrix has at least one dim of length 0
-            // so we do this loop, which either iterates over 0 elements
-            // or else it does the right thing for length-0 A.
-            dtype_%(z)s * zptr = (dtype_%(z)s*)(PyArray_DATA(%(z)s));
-            for (int i = 0; i < NA0; ++i)
-            {
-                zptr[i * Sz] = (dbeta == 0.0 ? 0.0 : zptr[i * Sz] * dbeta);
-            }
-        }
     }
     """
     return code % locals()
@@ -597,21 +592,21 @@ class CGemv(BaseBLAS, Gemv):
             alpha,
             beta,
             fail=sub["fail"],
-            force_init_beta=check_force_gemv_init(),
+            must_initialize_y=must_initialize_y_gemv(),
             params=sub["params"],
         )
         return code
 
     def c_code_cache_version(self):
-        return (14, blas_header_version(), check_force_gemv_init())
+        return (17, blas_header_version(), must_initialize_y_gemv())
 
 
 cgemv_inplace = CGemv(inplace=True)
 cgemv_no_inplace = CGemv(inplace=False)
 
 
-def check_force_gemv_init():
-    if check_force_gemv_init._force_init_beta is None:
+def must_initialize_y_gemv():
+    if must_initialize_y_gemv._force_init_beta is None:
         from pytensor.link.c.cmodule import GCC_compiler
 
         """
@@ -657,13 +652,13 @@ int main() {
         )
         if res:
             if res[0]:
-                check_force_gemv_init._force_init_beta = res[1]
+                must_initialize_y_gemv._force_init_beta = res[1]
             else:
-                check_force_gemv_init._force_init_beta = False
+                must_initialize_y_gemv._force_init_beta = False
         else:
-            check_force_gemv_init._force_init_beta = False
+            must_initialize_y_gemv._force_init_beta = False
 
-    return check_force_gemv_init._force_init_beta
+    return must_initialize_y_gemv._force_init_beta
 
 
-check_force_gemv_init._force_init_beta = None
+must_initialize_y_gemv._force_init_beta = None
