@@ -150,7 +150,9 @@ class TestConfigTypes:
             cp._apply("cuda123")
         with pytest.raises(ValueError, match="It was removed from PyTensor"):
             cp._apply("gpu123")
-        with pytest.raises(ValueError, match='Valid options start with one of "cpu".'):
+        with pytest.raises(
+            ValueError, match='Valid options start with one of "cpu"\\.'
+        ):
             cp._apply("notadevice")
         assert str(cp) == "unnamed (cpu)"
 
@@ -233,9 +235,9 @@ def test_config_pickling():
     for name in root._config_var_dict:
         v_original = getattr(root, name)
         v_restored = getattr(restored, name)
-        assert (
-            v_restored == v_original
-        ), f"{name} did not survive pickling ({v_restored} != {v_original})"
+        assert v_restored == v_original, (
+            f"{name} did not survive pickling ({v_restored} != {v_original})"
+        )
 
     # and validate that the test would catch typical problems
     root = _create_test_config()
@@ -247,7 +249,7 @@ def test_config_pickling():
     )
     with pytest.raises(
         AttributeError,
-        match="Can't (pickle|get) local object 'test_config_pickling.<locals>.<lambda>'",
+        match=r"Can't (pickle|get) local object 'test_config_pickling\.<locals>\.<lambda>'",
     ):
         pickle.dump(root, io.BytesIO())
 
