@@ -28,11 +28,12 @@ from pytensor.gradient import (
     zero_grad,
     zero_grad_,
 )
-from pytensor.graph.basic import Apply, graph_inputs
+from pytensor.graph.basic import Apply
 from pytensor.graph.null_type import NullType
 from pytensor.graph.op import Op
+from pytensor.graph.traversal import graph_inputs
 from pytensor.scan.op import Scan
-from pytensor.tensor.math import add, dot, exp, outer, sigmoid, sqr, tanh
+from pytensor.tensor.math import add, dot, exp, outer, sigmoid, sqr, sqrt, tanh
 from pytensor.tensor.math import sum as pt_sum
 from pytensor.tensor.random import RandomStream
 from pytensor.tensor.type import (
@@ -87,8 +88,8 @@ class TestGradSourcesInputs:
                 return Apply(self, inputs, outputs)
 
             def grad(self, inp, grads):
-                (x,) = inp
-                (gz,) = grads
+                (_x,) = inp
+                (_gz,) = grads
 
             def perform(self, *args, **kwargs):
                 raise NotImplementedError()
@@ -156,8 +157,8 @@ class TestGradSourcesInputs:
                 return Apply(self, inputs, outputs)
 
             def grad(self, inp, grads):
-                (x,) = inp
-                gz1, gz2 = grads
+                (_x,) = inp
+                _gz1, _gz2 = grads
                 return (gval,)
 
             def perform(self, *args, **kwargs):
@@ -181,8 +182,8 @@ class TestGradSourcesInputs:
                 return Apply(self, inputs, outputs)
 
             def grad(self, inp, grads):
-                x0, x1 = inp
-                (gz,) = grads
+                _x0, _x1 = inp
+                (_gz,) = grads
                 return (gval0, gval1)
 
             def perform(self, *args, **kwargs):
@@ -230,8 +231,8 @@ class TestGrad:
             return Apply(self, inputs, outputs)
 
         def grad(self, inp, grads):
-            x0, x1 = inp
-            gz0, gz1 = grads
+            _x0, _x1 = inp
+            _gz0, _gz1 = grads
             return self.gval0, self.gval1
 
         def perform(self, *args, **kwargs):
@@ -1143,6 +1144,24 @@ class TestJacobian:
         fn = function([x], jac_y, trust_input=True)
         benchmark(fn, np.array([0, 1, 2], dtype=x.type.dtype))
 
+    def test_benchmark_partial_jacobian(self, vectorize, benchmark):
+        # Example from https://github.com/jax-ml/jax/discussions/5904#discussioncomment-422956
+        N = 1000
+        rng = np.random.default_rng(2025)
+        x_test = rng.random((N,))
+
+        f_mat = rng.random((N, N))
+        x = vector("x", dtype="float64")
+
+        def f(x):
+            return sqrt(f_mat @ x / N)
+
+        full_jacobian = jacobian(f(x), x, vectorize=vectorize)
+        partial_jacobian = full_jacobian[:5, :5]
+
+        f = pytensor.function([x], partial_jacobian, trust_input=True)
+        benchmark(f, x_test)
+
 
 def test_hessian():
     x = vector()
